/*
    Gnu-Ising is a program to simulate the 2D stochastic Ising model.
    Copyright (C) 2001 Raphael Cerf

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    E-mail contact: Raphael.Cerf@math.u-psud.fr
    Ordinary mail: 
	Raphael Cerf, Mathematique, batiment 425
	Universite Paris Sud, 91405 Orsay Cedex

*/

#include <unistd.h>
#include <gtk--/accelgroup.h>
#include <gtk--.h>
#include <gtk--/main.h>
#include <gtk--/button.h>
#include <gtk--/style.h>
#include <gtk--/box.h>
#include <gtk--/drawingarea.h>
#include <gtk--/adjustment.h>
#include <gtk--/scale.h>
#include <gtk--/box.h>
#include <gtk--/frame.h>
#include <gtk--/label.h>
#include <gtk--/spinbutton.h>
#include <gtk--/adjustment.h>
#include <gtk--/window.h>
#include <gtk--/separator.h>
#include <gtk--/scrollbar.h>
#include <gtk--/checkbutton.h>
#include <string>
#include <gtk--/menu.h>
#include <gtk--/menubar.h>
#include <gtk--/optionmenu.h>
#include <gtk--/separator.h>
#include <gtk--/radiobutton.h>
#include <math.h>


#include <stdio.h>
#include <getopt.h>
#include <time.h>
#include <limits.h>
#include <vector.h>
#include <iterator.h>
#include <algo.h>
#include <iostream.h>
#include <math.h>
#include "MersenneTwister.h"

#define NTHREADS 10

enum algo_t {Glauber, Glauber_with_field, Glauber_with_random_field, Kawasaki};

namespace computation {

enum bc_t {free, minus, plus, toric, Dobrushin, diagonal};

class Fields {
	MTRand mtrand;
	float *fields;
	int hindex_max, vindex_max;
public:
	Fields();
	~Fields();
	int hsize, vsize;
	void init(int h, int v);
	inline float field(int i, int j) const;
};

Fields::Fields() {
	fields=new(float);
}

Fields::~Fields() {
	delete [] fields;	
}

void Fields::init(int h, int v) {
	float *f;

	hsize=h;
	vsize=v;
	hindex_max=h-1;
	vindex_max=v-1;
	delete [] fields;	
	fields=new float[h*v];

	f=fields;

	for (int i=0; i<hsize; i++)
		for (int j=0; j<vsize; j++)
			*f++= ( 2.*mtrand.rand()-1.);
}

inline float Fields::field(int i, int j) const 
{ return *(fields+(i*vsize+j)); }

class Sites {
	MTRand mtrand_hindex;
	MTRand mtrand_vindex;
	MTRand mtrand_flip;
	MTRand mtrand_hindex0;
	MTRand mtrand_vindex0;
	MTRand mtrand_flip0;
	MTRand mtrand_hindex1;
	MTRand mtrand_vindex1;
	MTRand mtrand_flip1;
	bool *sites;
	int hindex_max, vindex_max;
	bc_t bc;
public:
	int hsize, vsize;
	float prob, temp;
	float field, epsi;
	float expm2beta, expm4beta, expm6beta, expm8beta, expm12beta;
	float expmbeta[10];
	Sites();
	~Sites();

	void init(int h, int v);
	void set_bc(int num);
	bool site_status(int i, int j) const;
	bool site(int i, int j) const;
	int true_neighbours(int i, int j) const;
	int delta_neighbours(int i, int j) const;
	void copy(int i, int j, int k, int l);
	int bc_hindex(int i);
	int bc_vindex(int i);
	int random_hindex(); 
	int random_vindex(); 
	int random_hindex0(); 
	int random_vindex0(); 
	int random_hindex1(); 
	int random_vindex1(); 
	void set_prob(float p);
	void set_temp(float T);
	void set_field(float h, float e);
	void set_site(int i, int j, bool v);
	void flip_random_site();
	void exchange_sites(int i1, int j1, int i2, int j2);
	void flip_site(int i, int j);
	void flip_fair_site(int i, int j);
	void flip_unfair_site(int i, int j, float p);
	void flip_fair_site0(int i, int j);
	void flip_unfair_site0(int i, int j, float p);
	void flip_fair_site1(int i, int j);
	void flip_unfair_site1(int i, int j, float p);
	void print();
	void report();
};

class LatticeAlgo : public Sites {
	int *neighbours;
	int delay_report; 
	int seed;
	int iteration;
	int iteration0;
	int iteration1;
	MTRand mtrand_algo;
	class Fields fields;
public:
	LatticeAlgo();
	~LatticeAlgo();
	void init(int h, int v, float p, float T, float field=0, float epsi=0,
		int d_r=0, int se=4375U);
	int run(algo_t alg, int id=1, int max_iteration=-1);
	int glauber0(int id=1, int max_iteration=-1);
	int glauber1(int id=1, int max_iteration=-1);
	int glauber_with_field0(int id=1, int max_iteration=-1);
	int glauber_with_field1(int id=1, int max_iteration=-1);
	int glauber_with_random_field0(int id=1, int max_iteration=-1);
	int glauber_with_random_field1(int id=1, int max_iteration=-1);
	int kawasaki0(int id=1, int max_iteration=-1);
	int kawasaki1(int id=1, int max_iteration=-1);
	void set_neighbours(int i, int j, int v);
	int get_neighbours(int i, int j);
	void increase_neighbours(int i, int j);
	void decrease_neighbours(int i, int j);
	void increase_neighbours_of_site(int i, int j);
	void decrease_neighbours_of_site(int i, int j);
	void set_prob(float p);
	int get_iteration();
	void set_temp(float T);
	void set_field(float h, float e);
};

Sites::Sites() {
	sites=new(bool);
}

Sites::~Sites() {
	delete [] sites;	
}

void Sites::set_prob(float p) {
	prob=p;
}

void Sites::set_temp(float T) {
	temp=T;
}

void Sites::set_field(float h, float e) {
	field=h;
	epsi=e;
}

void Sites::set_bc(int num) {
	switch(num) {
		case 0: bc=free;
		break;
		case 1: bc=minus;
		break;
		case 2: bc=plus;
		break;
		case 3: bc=toric;
		break;
		case 4: bc=Dobrushin;
		break;
		case 5: bc=diagonal;
		break;
	}
}

inline void Sites::flip_site(int i, int j) {
*(sites+((vsize*i+j)))= !  *(sites+((vsize*i+j)));
}

inline void Sites::exchange_sites(int i1, int j1, int i2, int j2) {
static bool v;
v= *(sites+((vsize*i1+j1)));
*(sites+((vsize*i1+j1)))=  *(sites+((vsize*i2+j2)));
*(sites+((vsize*i2+j2)))=v;
}

inline void Sites::flip_random_site() {
*(sites+((vsize*mtrand_hindex.randInt(hindex_max)+mtrand_vindex.randInt(vindex_max))))= ( mtrand_flip.rand() < prob );
}

inline void Sites::flip_fair_site(int i, int j) {
*(sites+((vsize*i+j)))= ( mtrand_flip.rand() < 0.5 );
}

inline void Sites::flip_unfair_site(int i, int j, float p) {
if ( mtrand_flip.rand() < p )
	*(sites+((vsize*i+j)))= !*(sites+((vsize*i+j)));
}

inline void Sites::flip_fair_site0(int i, int j) {
*(sites+((vsize*i+j)))= ( mtrand_flip0.rand() < 0.5 );
}

inline void Sites::flip_unfair_site0(int i, int j, float p) {
if ( mtrand_flip0.rand() < p )
	*(sites+((vsize*i+j)))= !*(sites+((vsize*i+j)));
}

inline void Sites::flip_fair_site1(int i, int j) {
*(sites+((vsize*i+j)))= ( mtrand_flip1.rand() < 0.5 );
}

inline void Sites::flip_unfair_site1(int i, int j, float p) {
if ( mtrand_flip1.rand() < p )
	*(sites+((vsize*i+j)))= !*(sites+((vsize*i+j)));
}

inline bool Sites::site_status(int i, int j) const 
{ return *(sites+(i*vsize+j)); }
	
inline bool Sites::site(int i, int j) const 
{ return *(sites+(i*vsize+j)); }
	
inline int Sites::true_neighbours(int i, int j) const
{
	int sum=0;

	if (i>0) { if (site(i-1,j)) sum++;	}
	else { 
	
	switch(bc) {
	case free:
	break;
	case minus:
	break;
	case plus: sum++;
	break;
	case toric:
		if (site(hindex_max,j)) sum++;
	break;
	case Dobrushin:
		if (j<vindex_max/2) sum++;
	break;
	case diagonal: sum++;
	break;
	}

	}

	if (i<hindex_max) { if (site(i+1,j)) sum++;	}
	else { 
	
	switch(bc) {
	case free:
	break;
	case minus:
	break;
	case plus: sum++;
	break;
	case toric:
		if (site(0,j)) sum++;
	break;
	case Dobrushin:
		if (j<vindex_max/2) sum++;
	break;
	case diagonal: 
	break;
	}

	}

	if (j>0) { if (site(i,j-1)) sum++;	}
	else { 
	
	switch(bc) {
	case free:
	break;
	case minus:
	break;
	case plus: sum++;
	break;
	case toric:
		if (site(i,vindex_max)) sum++;
	break;
	case Dobrushin: sum++;
	break;
	case diagonal: sum++;
	break;
	}

	}

	if (j<vindex_max) { if (site(i,j+1)) sum++;	}
	else { 
	
	switch(bc) {
	case free:
	break;
	case minus:
	break;
	case plus: sum++;
	break;
	case toric:
		if (site(i,0)) sum++;
	break;
	case Dobrushin: 
	break;
	case diagonal: 
	break;
	}

	}

	return sum;
}

inline int Sites::delta_neighbours(int i, int j) const
{
	int delta=0;

	if (i>0) { delta+= (site(i-1,j) ? 1 : -1);	}
	else { 
	
	switch(bc) {
	case free:
	break;
	case minus: delta--;
	break;
	case plus: delta++;
	break;
	case toric:
	delta+= (site(hindex_max,j) ? 1 : -1);	
	break;
	case Dobrushin:
		delta+= ((j<vindex_max/2) ? 1: -1);
	break;
	case diagonal: delta++;
	break;
	}

	}

	if (i<hindex_max) { delta+= (site(i+1,j) ? 1 : -1); }	
	else { 
	
	switch(bc) {
	case free:
	break;
	case minus: delta--;
	break;
	case plus: delta++;
	break;
	case toric:
		delta+= (site(0,j) ? 1 : -1);	
	break;
	case Dobrushin:
		delta+= ((j<vindex_max/2) ? 1 : -1);	
	break;
	case diagonal: delta--;
	break;
	}

	}

	if (j>0) { delta+= (site(i,j-1) ? 1 : -1); }
	else { 
	
	switch(bc) {
	case free:
	break;
	case minus:delta--;
	break;
	case plus: delta++;
	break;
	case toric:
	delta+= (site(i,vindex_max) ? 1 : -1);	
	break;
	case Dobrushin: delta++;
	break;
	case diagonal: delta++;
	break;
	}

	}

	if (j<vindex_max) { delta+= (site(i,j+1) ? 1 : -1); }
	else { 
	
	switch(bc) {
	case free:
	break;
	case minus: delta--;
	break;
	case plus: delta++;
	break;
	case toric:
		delta+= (site(i,0) ? 1 : -1);	
	break;
	case Dobrushin: delta--;
	break;
	case diagonal: delta--;
	break;
	}

	}

	return delta;
}

inline void Sites::copy(int i, int j, int k, int l) {
*(sites+(k*vsize+l))= *(sites+(i*vsize+j));
}

inline int Sites::bc_hindex(int i)
{ 
	if (i==-1) return(hindex_max);
  	else if (i==hsize) return (0);
  	else return(i);
}

inline int Sites::bc_vindex(int j)
{ 
	if (j==-1) return(vindex_max);
  	else if (j==vsize) return (0);
  	else return(j);
}

inline int Sites::random_hindex() 
{ return mtrand_hindex.randInt(hindex_max); }

inline int Sites::random_vindex() 
{ return mtrand_vindex.randInt(vindex_max); }

inline int Sites::random_hindex0() 
{ return mtrand_hindex0.randInt(hindex_max); }

inline int Sites::random_vindex0() 
{ return mtrand_vindex0.randInt(vindex_max); }

inline int Sites::random_hindex1() 
{ return mtrand_hindex1.randInt(hindex_max); }

inline int Sites::random_vindex1() 
{ return mtrand_vindex1.randInt(vindex_max); }

inline void Sites::set_site(int i, int j, bool v)
{ *(sites+(i*vsize+j))=v; }

void Sites::init(int h, int v) {
	bool *s;

	hsize=h;
	vsize=v;
	hindex_max=h-1;
	vindex_max=v-1;
	delete [] sites;	
	sites=new bool[h*v];

	mtrand_hindex.seed( time(NULL));
	mtrand_vindex.seed( time(NULL)+1);
	mtrand_flip.seed( time(NULL)+3);

	mtrand_hindex0.seed( time(NULL)+4);
	mtrand_vindex0.seed( time(NULL)+5);
	mtrand_flip0.seed( time(NULL)+7);

	mtrand_hindex1.seed( time(NULL)+7);
	mtrand_vindex1.seed( time(NULL)+8);
	mtrand_flip1.seed( time(NULL)+10);

	s=sites;

	for (int i=0; i<hsize; i++)
		for (int j=0; j<vsize; j++)
			*s++= ( mtrand_flip.rand() < prob );
}

void Sites::print() {
	for (int i=0; i<hsize; i++) {
		for (int j=0; j<vsize; j++) {
        printf( "%d  ", *(sites+(i*vsize+j)));
		}
		cout << '\n';
	}
}

void Sites::report() {
	int histo[2]={0,0};

	for (int i=0; i<hsize; i++) {
		for (int j=0; j<vsize; j++) {
       			histo[(int)*(sites+(i*vsize+j))]++;
		}
	}
	for (int k=0; k<2; k++)
		if (histo[k]>0)
			cout << 100*histo[k]/hsize/vsize << ' ';
		else  cout << "  ";
	cout << '\n';

	if (hsize+vsize<50) { 
		print();
		cout << '\n';
	}
}


LatticeAlgo::~LatticeAlgo()
{
	delete [] neighbours;	
}

LatticeAlgo::LatticeAlgo()
{ neighbours=new (int); }

inline void LatticeAlgo::set_neighbours(int i, int j, int v)
{
	*(neighbours+(i*vsize+j))=v;
}

int LatticeAlgo::get_iteration() 
{ return iteration+iteration0+iteration1; }

inline int LatticeAlgo::get_neighbours(int i, int j)
{
	return(	*(neighbours+(i*vsize+j)));
}

inline void LatticeAlgo::decrease_neighbours(int i, int j)
{
	(*(neighbours+(i*vsize+j)))--;
}

inline void LatticeAlgo::increase_neighbours(int i, int j)
{
	(*(neighbours+(i*vsize+j)))++;
}

inline void LatticeAlgo::increase_neighbours_of_site(int i, int j)
{
	increase_neighbours(bc_hindex(i-1),j);
	increase_neighbours(bc_hindex(i+1),j);
	increase_neighbours(i,bc_vindex(j-1));
	increase_neighbours(i,bc_vindex(j+1));
}

inline void LatticeAlgo::decrease_neighbours_of_site(int i, int j) 
{
	decrease_neighbours(bc_hindex(i-1),j);
	decrease_neighbours(bc_hindex(i+1),j);
	decrease_neighbours(i,bc_vindex(j-1));
	decrease_neighbours(i,bc_vindex(j+1));
}

void LatticeAlgo::init(int h, int v, float p, float T, float hm, float e,
int d_r, int se)
{
	iteration=0;
	hsize=h; 
	vsize=v; 
	set_prob(p);
	set_temp(T);
	set_field(hm,e);
	delay_report=d_r;
	seed=se;
	mtrand_algo.seed( time(NULL));

	delete [] neighbours;
	neighbours=new int[hsize*vsize];
	Sites::init(hsize,vsize);
	fields.init(hsize,vsize);
	
	for (int i=0; i<hsize; i++) {
		for (int j=0; j<vsize; j++) {
		set_neighbours(i,j,true_neighbours(i,j));
	} }
}

void LatticeAlgo::set_prob(float p) {
	prob=p;
}

void LatticeAlgo::set_temp(float T) {
	temp=T;
	expm2beta=exp(-2.0/temp);
	expm4beta=exp(-4.0/temp);
	expm6beta=exp(-6.0/temp);
	expm8beta=exp(-8.0/temp);
	expm12beta=exp(-12.0/temp);
	for (int i=0; i<10; i++) 
		expmbeta[i]= exp(-(float)i/temp);
}

void LatticeAlgo::set_field(float hm, float e) {
	field=hm;
	epsi=e;
}


int LatticeAlgo::run(algo_t al, int id, int max_iteration=-1) {
	switch(al) {
		case Glauber: if (id==0) glauber0(id, max_iteration);
			else if (id==1) glauber1(id, max_iteration);
			break;
		case Kawasaki: if (id==0) kawasaki0(id, max_iteration);
			else if (id==1) kawasaki1(id, max_iteration);
			break;
		case Glauber_with_field: 
			if (id==0) glauber_with_field0(id, max_iteration);
			else if (id==1) glauber_with_field1(id, max_iteration);
			break;
		case Glauber_with_random_field: 
		if (id==0) glauber_with_random_field0(id, max_iteration);
		else if (id==1) glauber_with_random_field1(id, max_iteration);
			break;
	}
	return 0;
}


int LatticeAlgo::glauber0(int id, int max_iteration=-1) {
	int i,j;
	int  k;
	int delta;

	iteration0=0;
	
	while (iteration0!=max_iteration) {
		iteration0++;

	for (k=hsize*vsize; k>0; k--) {

		i=random_hindex0();
		j=random_vindex0();

 //first version of Glauber dynamics, without additional neighbours array 
		delta=delta_neighbours(i,j);
		if (delta==0) {
			flip_site(i,j);
		} else if (delta<0 && site(i,j)) {
			flip_site(i,j);
		} else if (delta>0 && !site(i,j)) {
			flip_site(i,j);
		// this part was the 0 temperature dynamics 
		} else if (delta==2 || delta==-2) {
			flip_unfair_site0(i, j, expm4beta); 
		} else if (delta==4 || delta==-4) {
			flip_unfair_site0(i, j, expm8beta); 
		}


		if (delay_report>0)
		if (iteration0% delay_report==0) {
			 	Sites::report();
		}

	} }
	
	cout << '\n';
	return(iteration0);
}

int LatticeAlgo::glauber1(int id, int max_iteration=-1) {
	int i,j;
	int  k;
	int delta;

	iteration1=0;
	
	while (iteration1!=max_iteration) {
		iteration1++;

	for (k=hsize*vsize; k>0; k--) {

		i=random_hindex1();
		j=random_vindex1();

 //first version of Glauber dynamics, without additional neighbours array 
		delta=delta_neighbours(i,j);
		if (delta==0) {
			flip_site(i,j);
		} else if (delta<0 && site(i,j)) {
			flip_site(i,j);
		} else if (delta>0 && !site(i,j)) {
			flip_site(i,j);
		// this part was the 0 temperature dynamics 
		} else if (delta==2 || delta==-2) {
			flip_unfair_site1(i, j, expm4beta); 
		} else if (delta==4 || delta==-4) {
			flip_unfair_site1(i, j, expm8beta); 
		}


		if (delay_report>0)
		if (iteration1% delay_report==0) {
			 	Sites::report();
		}

	} }
	
	cout << '\n';
	return(iteration1);
}


int LatticeAlgo::kawasaki0(int id, int max_iteration=-1) {
	int i1=0,j1=0;
	int i2=0,j2=0;
	int  k;
	int ex,delta;
	int delta1, delta2;
	MTRand mtrand;

	mtrand.seed(time(NULL));

	iteration0=0;
	
	while (iteration0!=max_iteration) {
		iteration0++;

	for (k=hsize*vsize; k>0; k--) {

		i1=random_hindex0();
		j1=random_vindex0();

		i2=random_hindex0();
		j2=random_vindex0();

		/*if (mtrand.rand()<0.5) {
			if (i1<hsize-1) i2=i1+1;
			else i2=i1;
			j2=j1;
		} else {
			i2=i1;
			if (j1<vsize-1) j2=j1+1;
			else j2=j1;
		}*/

		ex= (site(i1,j1) ? 1 : -1) -(site(i2,j2) ? 1 : -1);

		if (ex!=0) {

		delta1=delta_neighbours(i1,j1);
		delta2=delta_neighbours(i2,j2);

		//delta=ex*(delta1-delta2+ex);
		delta=ex*(delta1-delta2);
		/* this computation is wrong in case the two sites
			chosen randomly happened to be neighbours */
		if (delta<=0) {
			exchange_sites(i1,j1,i2,j2);
		// this part was the 0 temperature dynamics 
		} else if (mtrand.rand()<exp(-delta/temp)) {
				exchange_sites(i1,j1,i2,j2);
			}
		}
	
		if (delay_report>0)
		if (iteration0% delay_report==0) {
			 	Sites::report();
		} 
	} }
	
	cout << '\n';
	return(iteration0);
}

int LatticeAlgo::kawasaki1(int id, int max_iteration=-1) {
	int i1=0,j1=0;
	int i2=0,j2=0;
	int  k;
	int ex,delta;
	int delta1, delta2;
	MTRand mtrand;

	mtrand.seed(time(NULL));

	iteration1=0;
	
	while (iteration1!=max_iteration) {
		iteration1++;

	for (k=hsize*vsize; k>0; k--) {

		i1=random_hindex1();
		j1=random_vindex1();

		i2=random_hindex1();
		j2=random_vindex1();

		/*if (mtrand.rand()<0.5) {
			if (i1<hsize-1) i2=i1+1;
			else i2=i1;
			j2=j1;
		} else {
			i2=i1;
			if (j1<vsize-1) j2=j1+1;
			else j2=j1;
		}*/

		ex= (site(i1,j1) ? 1 : -1) -(site(i2,j2) ? 1 : -1);

		if (ex!=0) {

		delta1=delta_neighbours(i1,j1);
		delta2=delta_neighbours(i2,j2);

		//delta=ex*(delta1-delta2+ex);
		delta=ex*(delta1-delta2);
		/* this computation is wrong in case the two sites
			chosen randomly happened to be neighbours */
		if (delta<=0) {
			exchange_sites(i1,j1,i2,j2);
		// this part was the 0 temperature dynamics 
		} else if (mtrand.rand()<exp(-delta/temp)) {
				exchange_sites(i1,j1,i2,j2);
			}
		}
	
		if (delay_report>0)
		if (iteration1% delay_report==0) {
			 	Sites::report();
		} 
	} }
	
	cout << '\n';
	return(iteration1);
}

int LatticeAlgo::glauber_with_field0(int id, int max_iteration=-1) {
	int i,j;
	int  k;
	float deltaE;

	iteration0=0;
	
	while (iteration0!=max_iteration) {
		iteration0++;

	for (k=hsize*vsize; k>0; k--) {

		i=random_hindex0();
		j=random_vindex0();

	deltaE=((float)delta_neighbours(i,j)+field)*(site(i,j)? 2. : -2.);
		if (deltaE<=0) flip_site(i,j);
		else  flip_unfair_site0(i, j, exp(-deltaE/temp));

		if (delay_report>0)
		if (iteration0% delay_report==0) {
			 	Sites::report();
		}

	} }
	
	cout << '\n';
	return(iteration0);
}

int LatticeAlgo::glauber_with_field1(int id, int max_iteration=-1) {
	int i,j;
	int  k;
	float deltaE;

	iteration1=0;
	
	while (iteration1!=max_iteration) {
		iteration1++;

	for (k=hsize*vsize; k>0; k--) {

		i=random_hindex1();
		j=random_vindex1();

	deltaE=((float)delta_neighbours(i,j)+field)*(site(i,j)? 2. : -2.);
		if (deltaE<=0) flip_site(i,j);
		else  flip_unfair_site1(i, j, exp(-deltaE/temp));

		if (delay_report>0)
		if (iteration1% delay_report==0) {
			 	Sites::report();
		}

	} }
	
	cout << '\n';
	return(iteration1);
}

int LatticeAlgo::glauber_with_random_field0(int id, int max_iteration=-1) {
	int i,j;
	int  k;
	float deltaE;

	iteration0=0;
	
	while (iteration0!=max_iteration) {
		iteration0++;

	for (k=hsize*vsize; k>0; k--) {

		i=random_hindex0();
		j=random_vindex0();

	deltaE=((float)delta_neighbours(i,j)+
		epsi*fields.field(i,j))*(site(i,j)? 2. : -2.);
		if (deltaE<0) flip_site(i,j);
		else  flip_unfair_site0(i, j, exp(-deltaE/temp));

		if (delay_report>0)
		if (iteration0% delay_report==0) {
			 	Sites::report();
		}

	} }
	
	cout << '\n';
	return(iteration0);
}

int LatticeAlgo::glauber_with_random_field1(int id, int max_iteration=-1) {
	int i,j;
	int  k;
	float deltaE;

	iteration1=0;
	
	while (iteration1!=max_iteration) {
		iteration1++;

	for (k=hsize*vsize; k>0; k--) {

		i=random_hindex1();
		j=random_vindex1();

	deltaE=((float)delta_neighbours(i,j)+
		epsi*fields.field(i,j))*(site(i,j)? 2. : -2.);
		if (deltaE<0) flip_site(i,j);
		else  flip_unfair_site1(i, j, exp(-deltaE/temp));

		if (delay_report>0)
		if (iteration1% delay_report==0) {
			 	Sites::report();
		}

	} }
	
	cout << '\n';
	return(iteration1);
}

}



#include <pthread.h>

#define NTHREADS 10
pthread_t thread[NTHREADS];
typedef struct {
	int id;
	algo_t al;
} thread_info_t;
thread_info_t thread_info[NTHREADS];
int nthreads=0;

/* the algorithm */
class computation::LatticeAlgo algo;

#define errexit(code,str)                          \
  fprintf(stderr,"(%d) %s: %s\n",code,(str),strerror(code)); \
  exit(1);

/************************************************************************* 
	procedures to launch the threads runnning the simulation code 
**************************************************************************/

void *launch_computation(void *arg) {
  int id=((thread_info_t *) arg)->id;
  algo_t al=((thread_info_t *) arg)->al;
    pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
    pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
	algo.run(al,id);
   return ((void *)NULL);
}

int launch_computation_thread(algo_t al, int num) {
	for (int i=0; i<num; i++) {	
	if (nthreads<NTHREADS) {
    		thread_info[nthreads].id=nthreads;
    		thread_info[nthreads].al=al;
    		if (int errcode=pthread_create(&thread[nthreads], 
			NULL, launch_computation, &thread_info[nthreads])) { 
   		errexit(errcode,"pthread_create");
   		}
    		nthreads++;
	} }
	return nthreads;
}

int stop_computation_threads() {
	    pthread_cancel(thread[0]);
	for (int i=0; i<nthreads; i++) {
	    if (int errcode=pthread_cancel(thread[i])) { 
      		errexit(errcode,"pthread_cancel");
	} }
	nthreads=0;
	return nthreads;
}

namespace interface {

/* the global parameters handled by control_window and data_display */
int refresh_hsize=10;
int refresh_vsize=10;
int refresh_delay=0;
int refresh_number=0;
float prob=0.5;
float temp=2.2691854;
float field=0.0;
gint h_digits=1;
float epsi=0.0;
gint e_digits=1;
int hlength,vlength;
gfloat refresh_size_value=100;
time_t time_start=time(NULL);

string black_color_name="green";
string white_color_name="blue";

#define NCOLORS 10 
string color_names[NCOLORS]={"black", "white", "red", "blue", "green",
"orange", "purple", "yellow", "grey", "brown"};

string bdc_name="free";
#define NBDC 5 
string bdc_names[NBDC]={"free", "minus", "plus", "toric", "Dobrushin"};

/***********************************************************************
   There are four main objects:
	DataDisplay: in charge of extracting and visualizing the data
	LatticeWindow: the window where the lattice is visualized
	LatticeDrawingArea: the visualized lattice
	ControlWindow: the user interface
************************************************************************/

class DataDisplay;

class LatticeDrawingArea  : public Gtk::DrawingArea
{
  /* Backing pixmap for drawing area */
  Gdk_Pixmap pixmap;
  Gdk_Image image;
  Gdk_GC gc;
  Gdk_GC black_gc;
  Gdk_GC white_gc;
  guint32 black_pixel, white_pixel;
  Gdk_Window win;
  Gdk_Visual visual;
  bool connected;
  DataDisplay *data_display;

  virtual gint configure_event_impl (GdkEventConfigure *event);
  virtual gint expose_event_impl (GdkEventExpose *event);
  virtual gint button_press_event_impl (GdkEventButton *event);
  virtual gint motion_notify_event_impl (GdkEventMotion *event);
  void draw_black_point (gint x, gint y);
  void draw_white_point (gint x, gint y);
  void put_black_pixel (gint x, gint y);
  void put_white_pixel (gint x, gint y);
  void draw_point (int x, int y, guint32 gray);
  void draw_image(gint x, gint y, gint w, gint h); 
  void set_connected_false();
  void set_connected_true();
public:
  LatticeDrawingArea (DataDisplay *d_d);
  ~LatticeDrawingArea ();
  void erase();
   void set_black_color();
   void set_white_color();
   friend class ControlWindow;
   friend class DataDisplay;
};


class LatticeWindow : public Gtk::Window
{
    Gtk::VBox vbox;
    DataDisplay *data_display;
  public:  
    LatticeDrawingArea drawing_area;
    LatticeWindow (DataDisplay *d_d);
    ~LatticeWindow ();
  }; 

class DataDisplay
{
    int connect_refresh_points_with_delay;
    int connect_refresh_points_without_delay;
    bool connected;
    MTRand mtrand;
    LatticeWindow lattice_window;
    bool control_window_ready;
public:
    ControlWindow *control_window;
    DataDisplay();
    ~DataDisplay();
    void init();
    void run();
    void restart();
    void stop();
    void erase();
    void show();
    void connect_with_delay();
    void connect_without_delay();
    void set_black_color();
    void set_white_color();
    void set_connected_false();
    void set_connected_true();
    void start_connection();
    void end_connection();
    void refresh_size_update();
    int DataDisplay::refresh_points_without_delay(); 
    int DataDisplay::refresh_points_with_delay(); 
   friend class ControlWindow;
};

class ControlWindow : public Gtk::Window
  {
    Gtk::SpinButton *spinner_T_val;
    Gtk::SpinButton *spinner_T_pre;
    Gtk::HScale *T_digits;
    Gtk::SpinButton *spinner_he_val;
    Gtk::SpinButton *spinner_he_pre;
    Gtk::HScale *he_digits;
    Gtk::SpinButton *spinner_p_val;
    Gtk::SpinButton *spinner_p_pre;
    Gtk::HScale *p_digits;
    Gtk::RadioButton rb_server, rb_client;
    Gtk::RadioButton rb_noise, rb_algo;
    Gtk::RadioButton rb_random, rb_sweep;
    Gtk::RadioButton rb_constant, rb_rndh;
    Gtk::CheckButton ck_field;
    Gtk::CheckButton ck_auto;
    Gtk::CheckButton ck_bipro;
    Gtk::RadioButton rb_glauber, rb_kawasaki;
    Gtk::Adjustment adj_size;
    Gtk::Adjustment adj_delay;
    Gtk::Button initer;
    Gtk::Button runner;
    Gtk::Button restarter;
    Gtk::Button stopper;
    Gtk::Button eraser;
    Gtk::Button button;
    Gtk::HScale *refresh_size_wt;
    Gtk::HScale *refresh_delay_wt;
    Gtk::Label *prob_label;
    Gtk::Frame *frame_stats;
    Gtk::Label *runtime_label;
    Gtk::Label *iter_label;
    Gtk::Label *refresh_label;
    Gtk::Label *minus_color_label;
    Gtk::Label *plus_color_label;
    Gtk::Label *bdc_label;
    Gtk::Label *para_label;
    Gtk::VBox vbox;
    Gtk::Connection connection;
    DataDisplay *data_display;
	
  public:  
    ControlWindow (DataDisplay *d_d);
    ~ControlWindow ();
    void set_button_states(bool i, bool r, bool t, bool s);
    void update_prob();
    void update_temp();
    void update_field();
    void p_digits_cb(Gtk::Adjustment *adj);
    void T_digits_cb(Gtk::Adjustment *adj);
    void he_digits_cb(Gtk::Adjustment *adj);
    void p_value_cb( Gtk::SpinButton *spin );
    void T_value_cb( Gtk::SpinButton *spin );
    void he_value_cb( Gtk::SpinButton *spin );
    void refresh_size_cb(Gtk::Adjustment *adj);
    void refresh_delay_cb(Gtk::Adjustment *adj);
    int statistics();
    void void_minus_color_int_cb(int num);
    void void_plus_color_int_cb(int num);
    void void_bdc_int_cb(int num);
    void field_cb();
    void constant_cb();
    void rndh_cb();
    void algo_cb();
    void run();
    void restart();
    void init();
    void stop();
    void erase();
    int refresh_points_without_delay();
    int refresh_points_with_delay(); 
    void start_connection();
    void end_connection();
   friend class DataDisplay;
  }; 


LatticeDrawingArea::LatticeDrawingArea (DataDisplay *d_d)
    : Gtk::DrawingArea(), pixmap (0)
  {
    set_events (GDK_EXPOSURE_MASK
		| GDK_LEAVE_NOTIFY_MASK
		| GDK_BUTTON_PRESS_MASK
		| GDK_POINTER_MOTION_MASK
		| GDK_POINTER_MOTION_HINT_MASK);
    data_display=d_d;
    connected=false;
  }

/***********************************************************************************
   LatticeDrawingArea
************************************************************************************/

LatticeDrawingArea::~LatticeDrawingArea()
  {}

void LatticeDrawingArea::set_connected_false() {
	connected=false;
}

void LatticeDrawingArea::set_connected_true() {
	connected=true;
}

/* Create a new backing pixmap of the appropriate size */
int LatticeDrawingArea::configure_event_impl (GdkEventConfigure *event)
  {
    data_display->end_connection();
    data_display->stop();

    win = get_window();
    visual = win.get_visual();

    if (pixmap)
      pixmap.release();

    gc = get_style()->get_white_gc();
    pixmap.create(get_window(),  width(), height());

    pixmap.draw_rectangle (gc, true, 0, 0, width(), height());

    hlength=width();
    vlength=height();

    black_gc = Gdk_GC(pixmap);
    Gdk_Color black(black_color_name);
    get_colormap().alloc(black);
    black_gc.set_foreground(black);
    black_pixel=black.get_pixel();

    white_gc = Gdk_GC(pixmap);
    Gdk_Color white(white_color_name);
    get_colormap().alloc(white);
    white_gc.set_foreground(white);
    white_pixel=white.get_pixel();

    image.create(GDK_IMAGE_NORMAL, visual, hlength, vlength);

    data_display->refresh_size_update();
    data_display->init();
    data_display->start_connection();
    data_display->run();
    return true;
  }

void LatticeDrawingArea::set_black_color() {
    black_gc = Gdk_GC(pixmap);
    Gdk_Color black(black_color_name);
    get_colormap().alloc(black);
    black_gc.set_foreground(black);
    black_pixel=black.get_pixel();
}

void LatticeDrawingArea::set_white_color() {
    white_gc = Gdk_GC(pixmap);
    Gdk_Color white(white_color_name);
    get_colormap().alloc(white);
    white_gc.set_foreground(white);
    white_pixel=white.get_pixel();
}

/* Redraw the screen from the backing pixmap */
int LatticeDrawingArea::expose_event_impl (GdkEventExpose *event)
  {
    gc = get_style()->get_fg_gc(get_state());
    win.draw_pixmap(gc ,
		    pixmap,
		    event->area.x, event->area.y,
		    event->area.x, event->area.y,
		    event->area.width, event->area.height);

    return false;
  }

inline void LatticeDrawingArea::draw_black_point (gint x, gint y)
  {
    pixmap.draw_point(black_gc,x,y);
  }

inline void LatticeDrawingArea::draw_white_point (gint x, gint y)
  {
    pixmap.draw_point(white_gc,x,y);
  }

inline void LatticeDrawingArea::put_black_pixel (gint x, gint y)
  {
    image.put_pixel(x,y,black_pixel);
  }

inline void LatticeDrawingArea::put_white_pixel (gint x, gint y)
  {
    image.put_pixel(x,y,white_pixel);
  }

void LatticeDrawingArea::draw_point (int x, int y, guint32 gray)
{}

inline void LatticeDrawingArea::draw_image(gint x, gint y, gint w, gint h) {
	pixmap.draw_image(gc, image, x, y, x, y, w, h);
}

gint LatticeDrawingArea::button_press_event_impl (GdkEventButton *event)
  {
    if (event->button == 1 && pixmap)
      draw_white_point (event->x, event->y);

    return true;
  }

gint LatticeDrawingArea::motion_notify_event_impl (GdkEventMotion *event)
  {
    int x, y;
    GdkModifierType state;
    Gdk_Window window(event->window);
    if (event->is_hint)
      window.get_pointer (x, y, state);
    else {
	x = (int)event->x;
	y = (int)event->y;
	state = (GdkModifierType) event->state;
      }
    
    if (state & GDK_BUTTON1_MASK && pixmap)
      draw_white_point (x, y);
  
    return true;
  }

void LatticeDrawingArea::erase()
  {
    // clear pixmap area to white.
    gc = get_style()->get_white_gc();
    pixmap.draw_rectangle (gc, true, 0, 0, width(), height());

    // request a refresh of whole area.
    draw(0);
  }

/***********************************************************************************
   LatticeWindow
************************************************************************************/

LatticeWindow::LatticeWindow (DataDisplay *d_d)
    :  Gtk::Window(GTK_WINDOW_TOPLEVEL),
	vbox (false, 0), drawing_area(d_d)
  {
    data_display=d_d;

    set_title("Lattice");
    add (vbox);

    vbox.pack_start (drawing_area, true, true, 0);

    destroy.connect(Gtk::Main::quit.slot());

    drawing_area.show();
    vbox.show();
  }

LatticeWindow::~LatticeWindow()
  {}

/***********************************************************************************
   ControlWindow
************************************************************************************/

void ControlWindow::run() {
	data_display->run();
}

void ControlWindow::restart() {
	data_display->restart();
}

void ControlWindow::init() {
	data_display->init();
}

void ControlWindow::stop() {
	data_display->stop();
}

void ControlWindow::erase() {
	data_display->erase();
}

void ControlWindow::T_value_cb( Gtk::SpinButton *spin )
{
	temp=spin->get_value_as_float (); 
	update_temp();
}

void ControlWindow::he_value_cb( Gtk::SpinButton *spin )
{
	if (rb_constant.get_active()) 
		{ field=spin->get_value_as_float(); }
	if (rb_rndh.get_active()) 
		{ epsi=spin->get_value_as_float(); }
	update_field();
}

void ControlWindow::p_value_cb( Gtk::SpinButton *spin )
{
	prob=spin->get_value_as_float (); 
	update_prob();
}

void ControlWindow::void_minus_color_int_cb(int num)
{
	black_color_name=color_names[num];
	minus_color_label->set_text ("is "+black_color_name);

	data_display->set_black_color();
}

void ControlWindow::void_plus_color_int_cb(int num)
{
	white_color_name=color_names[num];
  	plus_color_label->set_text ("is "+white_color_name);
	data_display->set_white_color();
}

void ControlWindow::algo_cb() {
  data_display->stop();
  data_display->run();
}

void ControlWindow::field_cb() {
	algo_cb();
	if (rb_constant.get_active()) { ; }
	if (rb_rndh.get_active()) { ; }
}

void ControlWindow::constant_cb() {
	para_label->set_text ("h=");
 	spinner_he_val->set_digits(h_digits);
(he_digits->get_adjustment())->set_value((gfloat)h_digits);
(spinner_he_val->get_adjustment())->set_step_increment(pow(10, (int)-h_digits));
(spinner_he_val->get_adjustment())->set_value(field);
	field_cb();
}

void ControlWindow::rndh_cb() {
	para_label->set_text ("e=");
 	spinner_he_val->set_digits(e_digits);
(he_digits->get_adjustment())->set_value((gfloat)e_digits);
(spinner_he_val->get_adjustment())->set_step_increment(pow(10, (int)-e_digits));
(spinner_he_val->get_adjustment())->set_value(epsi);
	field_cb();
}

void ControlWindow::void_bdc_int_cb(int num)
{
	bdc_name=bdc_names[num];
	bdc_label->set_text ("are "+bdc_name);
	algo.set_bc(num);
}

ControlWindow::ControlWindow (DataDisplay *d_d)
    :  Gtk::Window(GTK_WINDOW_TOPLEVEL),
       rb_server("server"),
       rb_client("client"),
       rb_noise("noise"),
       rb_algo("algo   "),
       rb_random("rand"),
       rb_sweep("sweep"),
       rb_constant("constant"),
       rb_rndh("random"),
       ck_field("field"),
       ck_auto("auto"),
       ck_bipro("bipro"),
       rb_glauber("Glauber"),
       rb_kawasaki("Kawasaki"),
	adj_size(100, 1, 100),
	adj_delay(0, 0, 100),
       initer("init"),
       runner("run"),
       restarter("restart"),
       stopper("stop"),
       eraser("erase"),
       button ("quit")
  {
  data_display=d_d;
  data_display->show();
  
  data_display->control_window=this;
  data_display->control_window_ready=true;

  set_title("Gnu Ising");

  Gtk::VBox *main_vbox = manage( new Gtk::VBox(false, 0) );
  add (*main_vbox);

  Gtk::Frame *frame_temp = manage( new Gtk::Frame ("T (Tc=2.2691854...)") );
  main_vbox->pack_start (*frame_temp, true, true, 10);

	  Gtk::HBox *box10=manage(new Gtk::HBox(false,10));
	  frame_temp->add (*box10);

	  Gtk::HBox *box101=manage(new Gtk::HBox(false,10));
	  box10->pack_start(*box101,false,false,0);

  	  	box101->pack_start (*manage( new Gtk::Label ("T=", 0, 0.5) )
		, false, true, 0);
  
		Gtk::Adjustment *adj = manage( new Gtk::Adjustment (2.2691854, 0.0, 10.0, 0.1, 0.1, 0.0) );

  		spinner_T_val = manage( new Gtk::SpinButton (*adj, 0, 1) );
  		spinner_T_val->set_wrap (false);
  		spinner_T_val->set_usize (65 , 0);
  		adj->value_changed.connect(bind(slot(this, &ControlWindow::T_value_cb),spinner_T_val));
  		box101->pack_start (*spinner_T_val, false, true, 0);

	  Gtk::HBox *box102=manage(new Gtk::HBox(false,10));
	  box10->pack_start(*box102,false,false,0);

	  	Gtk::HBox *box00=manage(new Gtk::HBox(false,0));
	  	box102->pack_start(*manage(new Gtk::Label("digits: ",0)),false,false);
	
	  	Gtk::Adjustment *adj1=manage(new Gtk::Adjustment(1, 1, 4));
	  	T_digits=manage(new Gtk::HScale(*adj1)); 
 	  	T_digits->set_digits(0);
	  	adj1->value_changed.connect(bind(slot(this,&ControlWindow::T_digits_cb),adj1));
	  	box102->pack_start(*T_digits,true,true);

  Gtk::Frame *frame_algo = manage( new Gtk::Frame ("dynamics") );
  main_vbox->pack_start (*frame_algo, true, true, 10);

	  Gtk::VBox *box_dyna=manage(new Gtk::VBox(false,0));
	  frame_algo-> add (*box_dyna);

	  Gtk::HBox *box_algo=manage(new Gtk::HBox(false,0));
	  box_dyna->pack_start(*box_algo);

		  rb_glauber.set_group(rb_kawasaki.group());
		  rb_glauber.set_active(true);
  rb_glauber.clicked.connect(slot(this,&ControlWindow::algo_cb));
  rb_kawasaki.clicked.connect(slot(this,&ControlWindow::algo_cb));

		  box_algo->pack_start(rb_glauber);
		  box_algo->pack_start(rb_kawasaki);

  box_dyna->pack_start (*manage(new Gtk::HSeparator()),false, true,0); 
	  Gtk::HBox *box99=manage(new Gtk::HBox(false,10));
	  box_dyna->pack_start(*box99);

  Gtk::MenuBar *bdc_menubar = manage( new Gtk::MenuBar());

  {
     using namespace Gtk;
     using namespace Menu_Helpers;

     Menu    *bdc_menu      = manage( new Menu());
     MenuList& bdc_list_file     = bdc_menu->items();

	for (int i=0; i<NBDC; i++) {
     bdc_list_file.push_back(MenuElem(bdc_names[i], bind(slot(this, &ControlWindow::void_bdc_int_cb), i)));
	}

     MenuElem bdc_elem("boundary conditions",*bdc_menu);
     bdc_menubar->items().push_front(bdc_elem);

   }
   box99->pack_start (*bdc_menubar, false, false);

  bdc_label = manage( new Gtk::Label ("") );
  bdc_label->set_text ("are "+bdc_name);
  box99->pack_start (*bdc_label, false, true, 0);

  box_dyna->pack_start (*manage(new Gtk::HSeparator()),false, true,0); 
	  Gtk::HBox *box88=manage(new Gtk::HBox(false,10));
	  box_dyna->pack_start(*box88);

	  ck_field.set_active(false);
	  box88->pack_start(ck_field);

  	  ck_field.clicked.connect(slot(this,&ControlWindow::field_cb));


	  box88->pack_start (*manage(new Gtk::VSeparator()),false, true,0); 
		  rb_constant.set_group(rb_rndh.group());
		  rb_constant.set_active(true);
	rb_constant.clicked.connect(slot(this,&ControlWindow::constant_cb));
	rb_rndh.clicked.connect(slot(this,&ControlWindow::rndh_cb));

		  Gtk::HBox *box825=manage(new Gtk::HBox(false,0));
	  box88->pack_start (*box825, false, false, 0); 

		  box825->pack_start(rb_constant);
		  box825->pack_start(rb_rndh);

  box_dyna->pack_start (*manage(new Gtk::HSeparator()),false, true,0); 
	  Gtk::HBox *box77=manage(new Gtk::HBox(false,10));
   box_dyna->pack_start(*box77);

	  Gtk::HBox *box771=manage(new Gtk::HBox(false,10));
	  box77->pack_start(*box771,false,false,0);

		para_label=manage( new Gtk::Label ("h=", 0, 0.5));
  	  	box771->pack_start (*para_label , false, true, 0);
  
		Gtk::Adjustment *adjh = manage( new Gtk::Adjustment (0.0, -8.0, 8.0, 0.1, 0.1, 0.0) );

  		spinner_he_val = manage( new Gtk::SpinButton (*adjh, 0, 1) );
  		spinner_he_val->set_wrap (false);
  		spinner_he_val->set_usize (65 , 0);
  		adjh->value_changed.connect(bind(slot(this, &ControlWindow::he_value_cb),spinner_he_val));
  		box771->pack_start (*spinner_he_val, false, true, 0);

	  Gtk::HBox *box772=manage(new Gtk::HBox(false,10));
	  box77->pack_start(*box772,false,false,0);

	  	Gtk::HBox *box700=manage(new Gtk::HBox(false,0));
	  	box772->pack_start(*manage(new Gtk::Label("digits: ",0)),false,false);
	
	  	Gtk::Adjustment *adjh1=manage(new Gtk::Adjustment(1, 1, 4));
	  	he_digits=manage(new Gtk::HScale(*adjh1)); 
 	  	he_digits->set_digits(0);
	  	adjh1->value_changed.connect(bind(slot(this,&ControlWindow::he_digits_cb),adjh1));
	  	box772->pack_start(*he_digits,true,true);


  Gtk::Frame *frame_prob = manage( 
		new Gtk::Frame ("p (initial Bernoulli configuration)") );
  main_vbox->pack_start (*frame_prob, true, true, 10);

	  Gtk::HBox *box11=manage(new Gtk::HBox(false,10));
	  frame_prob->add (*box11);

	  Gtk::HBox *box111=manage(new Gtk::HBox(false,10));
	  box11->pack_start(*box111,false,false,0);

  	  	box111->pack_start (*manage( new Gtk::Label ("p=", 0, 0.5) )
		, false, true, 0);
  
		Gtk::Adjustment *adjb = manage( new Gtk::Adjustment (0.5, 0.0, 1.0, 0.1, 0.1, 0.0) );

  		spinner_p_val = manage( new Gtk::SpinButton (*adjb, 0, 1) );
  		spinner_p_val->set_wrap (false);
  		spinner_p_val->set_usize (65 , 0);
  		adjb->value_changed.connect(bind(slot(this, &ControlWindow::p_value_cb),spinner_p_val));
  		box111->pack_start (*spinner_p_val, false, true, 0);

	  Gtk::HBox *box112=manage(new Gtk::HBox(false,10));
	  box11->pack_start(*box112,false,false,0);

	  	Gtk::HBox *box12=manage(new Gtk::HBox(false,0));
	  	box112->pack_start(*manage(new Gtk::Label("digits: ",0)),false,false);
	
	  	Gtk::Adjustment *adjb1=manage(new Gtk::Adjustment(1, 1, 4));
	  	p_digits=manage(new Gtk::HScale(*adjb1)); 
 	  	p_digits->set_digits(0);
	  	adjb1->value_changed.connect(bind(slot(this,&ControlWindow::p_digits_cb),adjb1));
	  	box112->pack_start(*p_digits,true,true);

  Gtk::Frame *frame_buttons = manage( new Gtk::Frame ("actions") );
  main_vbox->pack_start (*frame_buttons, true, true, 10);

  Gtk::HBox *hbox_buttons=manage(new Gtk::HBox(false,0));
  frame_buttons-> add (*hbox_buttons);

  hbox_buttons->pack_start (initer, false, false, 0); 
  hbox_buttons->pack_start (runner, false, false, 0); 
  hbox_buttons->pack_start (restarter, false, false, 0); 
  hbox_buttons->pack_start (stopper, false, false, 0); 
  hbox_buttons->pack_start (eraser, false, false, 0); 
  hbox_buttons->pack_start (button, false, false, 0); 

  set_button_states(true,false,false,false);

  initer.clicked.connect(slot(this, &ControlWindow::init)); 
  runner.clicked.connect(slot(this, &ControlWindow::run)); 
  restarter.clicked.connect(slot(this, &ControlWindow::restart)); 
  stopper.clicked.connect(slot(this, &ControlWindow::stop));

  eraser.clicked.connect(slot(this, &ControlWindow::erase));
  button.clicked.connect(destroy.slot());
  destroy.connect(Gtk::Main::quit.slot());

  Gtk::Frame *frame_refresh = manage( new Gtk::Frame ("refresh and display") );
  main_vbox->pack_start (*frame_refresh, true, true, 10);

	  data_display->connect_refresh_points_with_delay=0;
	  data_display->connect_refresh_points_without_delay=1;
	  start_connection();

	  Gtk::VBox *box_refresh=manage(new Gtk::VBox(false,0));
	  frame_refresh-> add (*box_refresh);


	     	Gtk::HBox *box21=manage(new Gtk::HBox(false,0));
	  	box_refresh->pack_start (*box21, false, false, 0); 

		  box21->pack_start(*manage(new Gtk::Label("size: ",0)),false,false);

		  refresh_size_value=100;
		  data_display->refresh_size_update();
		  refresh_size_wt=manage(new Gtk::HScale(adj_size)); 
		  refresh_size_wt->set_digits(0);
		  adj_size.value_changed.connect(bind(slot(this,&ControlWindow::refresh_size_cb),&adj_size));
		  box21->pack_start(*refresh_size_wt,true,true);

		  Gtk::HBox *box22=manage(new Gtk::HBox(false,0));
		  box_refresh->pack_start (*box22, false, false, 0); 

		  box22->pack_start(*manage(new Gtk::Label("delay: ",0)),false,false);

		  adj_delay.set_value(0);
		  refresh_delay=0;
		  refresh_delay_wt=manage(new Gtk::HScale(adj_delay)); 
		  refresh_delay_wt->set_digits(0);
		  adj_delay.value_changed.connect(bind(slot(this,&ControlWindow::refresh_delay_cb),&adj_delay));
		  box22->pack_start(*refresh_delay_wt,true,true);

	  box_refresh->pack_start (*manage(new Gtk::HSeparator),false, true); 

	  Gtk::HBox *box2_refresh=manage(new Gtk::HBox(false,0));
	  box_refresh->pack_start (*box2_refresh, false, false, 0); 


		  rb_client.set_group(rb_server.group());
		  rb_client.set_active(true);

		  Gtk::HBox *box23=manage(new Gtk::HBox(false,0));
		  box2_refresh->pack_start (*box23, false, false, 0); 

		  box23->pack_start(rb_client);
		  box23->pack_start(rb_server);

	  box2_refresh->pack_start (*manage(new Gtk::VSeparator()),false, true,0); 
		  rb_random.set_group(rb_sweep.group());
		  rb_sweep.set_active(true);

		  Gtk::HBox *box25=manage(new Gtk::HBox(false,0));
	  box2_refresh->pack_start (*box25, false, false, 0); 

		  box25->pack_start(rb_random);
		  box25->pack_start(rb_sweep);

  box_refresh->pack_start (*manage(new Gtk::HSeparator()),false, true,0); 

	  Gtk::HBox *box3_refresh=manage(new Gtk::HBox(false,0));
	  box_refresh->pack_start (*box3_refresh, false, false, 0); 

		  rb_noise.set_group(rb_algo.group());
		  rb_noise.set_active(true);

		  Gtk::HBox *box31=manage(new Gtk::HBox(false,0));
	  box3_refresh->pack_start (*box31, false, false, 0); 

		  box31->pack_start(rb_noise);
		  box31->pack_start(rb_algo);

	  box3_refresh->pack_start (*manage(new Gtk::VSeparator()),false, true,0); 
		  ck_auto.set_active(false);

		  Gtk::HBox *box32=manage(new Gtk::HBox(false,0));
	  box3_refresh->pack_start (*box32, false, false, 0); 

		  box32->pack_start(ck_auto);

		  Gtk::HBox *box311=manage(new Gtk::HBox(false,0));
	  box3_refresh->pack_start (*box311, false, false, 0); 

		  ck_bipro.set_active(false);
		  box311->pack_start(ck_bipro);
  ck_bipro.clicked.connect(slot(this,&ControlWindow::algo_cb));

    //hbox_buttons.pack_start (*manage(new Gtk::HSeparator),false, true, 0); 

  //Gtk::Frame *frame_display = manage( new Gtk::Frame ("display") );
  //main_vbox->pack_start (*frame_display, true, true, 10);

    box_refresh->pack_start (*manage(new Gtk::HSeparator),false, true, 0); 
	  Gtk::HBox *box_display=manage(new Gtk::HBox(false,0));
	  	box_refresh->pack_start(*box_display);


  Gtk::MenuBar *minus_menubar = manage( new Gtk::MenuBar());
  Gtk::MenuBar *plus_menubar  = manage( new Gtk::MenuBar());

 {
     using namespace Gtk;
     using namespace Menu_Helpers;

     Menu    *minus_menu_colors      = manage( new Menu());
     MenuList& minus_list_file     = minus_menu_colors->items();

	for (int i=0; i<NCOLORS; i++) {
     minus_list_file.push_back(MenuElem(color_names[i], bind(slot(this, &ControlWindow::void_minus_color_int_cb), i)));
	}

     Menu    *plus_menu_colors      = manage( new Menu());
     MenuList& plus_list_file     = plus_menu_colors->items();

	for (int i=0; i<NCOLORS; i++) {
     plus_list_file.push_back(MenuElem(color_names[i], bind(slot(this, &ControlWindow::void_plus_color_int_cb), i)));
	}

     MenuElem black_elem("minus",*minus_menu_colors);
     minus_menubar->items().push_front(black_elem);

     MenuElem plus_elem("plus",*plus_menu_colors);
     plus_menubar->items().push_front(plus_elem);

   }
   box_display->pack_start (*minus_menubar, false, false);

  minus_color_label = manage( new Gtk::Label ("") );
  minus_color_label->set_text ("is "+black_color_name);
  box_display->pack_start (*minus_color_label, false, true, 5);

  box_display->pack_start (*plus_menubar, false, false);

  plus_color_label = manage( new Gtk::Label ("") );
  plus_color_label->set_text ("is "+white_color_name);
  box_display->pack_start (*plus_color_label, false, true, 5);

		frame_stats = manage( new Gtk::Frame ("statistics") );
  main_vbox->pack_start (*frame_stats, true, true, 10);

	  Gtk::VBox *vbox_stats=manage(new Gtk::VBox(false,0));
	  frame_stats-> add (*vbox_stats);
	
	  Gtk::HBox *hbox1_stats=manage(new Gtk::HBox(false,0));
	  vbox_stats-> add (*hbox1_stats);
	
	  runtime_label = manage( new Gtk::Label ("") );
  	  hbox1_stats->pack_start (*runtime_label, false, true, 5);

	  refresh_label = manage( new Gtk::Label ("") );
  	  hbox1_stats->pack_start (*refresh_label, false, true, 5);

	  Gtk::HBox *hbox2_stats=manage(new Gtk::HBox(false,0));
	  vbox_stats-> add (*hbox2_stats);
	
	  iter_label = manage( new Gtk::Label ("") );
  	  hbox2_stats->pack_start (*iter_label, false, true, 5);

  Gtk::Main::timeout.connect(slot(this,&ControlWindow::statistics),1000);

  rb_algo.set_active(true);
  data_display->run();

  show_all();

  }

ControlWindow::~ControlWindow()
  {}

int ControlWindow::refresh_points_with_delay() 
{
	return(data_display->refresh_points_with_delay());
}

int ControlWindow::refresh_points_without_delay() 
{
	return(data_display->refresh_points_without_delay());
}

int ControlWindow::statistics() 
{
 	gchar buf[64];
	static time_t time1=time(NULL), time2;
	static int count=0;
	static int last_iteration=0;
	static int last_refresh=0;
	static int last_refresh_speed=0;
	static int iter_since_last_size_change=0;
	static int refresh_size_inc=-1;
	int refresh_speed;

	time2=time(NULL);
	float interval=(float)difftime(time2,time1);

    	sprintf (buf, "runtime:%ds", (int)difftime(time2,time_start));
  	runtime_label->set_text (buf);


	int iter=algo.get_iteration();
	if (last_iteration>iter) last_iteration=iter;

	if (interval>2.0) {

    	sprintf (buf, "statistics (%d)", count++);
  	frame_stats->set_label (buf);

    	sprintf (buf, "%d iterations (%.2f/s)", 
		iter, (float)(iter-last_iteration)/interval);
  	iter_label->set_text (buf);
	last_iteration=iter;

	float ratio=
		(float)(hlength*vlength)/(float)(refresh_hsize*refresh_vsize);
refresh_speed=(int)((float)(refresh_number-last_refresh)/interval/ratio);
	last_refresh=refresh_number;
	time1=time2;

	if  (refresh_delay==0) {

    	sprintf (buf, "refresh speed:%d/s", refresh_speed);
  	refresh_label->set_text (buf);
	
	if (ck_auto.get_active() && 
		++iter_since_last_size_change>1) {

	if (refresh_speed<last_refresh_speed) {
		refresh_size_inc=-refresh_size_inc;
	}
	last_refresh_speed=refresh_speed;
	refresh_size_value+=refresh_size_inc;
	iter_since_last_size_change=0;
	if (refresh_size_value,0) {
		refresh_size_value=0;
	}
	if (refresh_size_value>100) {
		refresh_size_value=100;
	}
	adj_size.set_value(refresh_size_value);
  	data_display->refresh_size_update();

	}
	
	} else {
    		sprintf (buf, " ");
  		refresh_label->set_text (buf);
	}

	}
	return (1);
}

void ControlWindow::set_button_states(bool i, bool r, bool t, bool s)
{
	initer.set_sensitive(i);
	runner.set_sensitive(r);
	restarter.set_sensitive(t);
	stopper.set_sensitive(s);
}

void ControlWindow::update_prob() {
	algo.set_prob(prob);
}

void ControlWindow::update_temp() {
	algo.set_temp(temp);
}

void ControlWindow::update_field() {
	algo.set_field(field,epsi);
}

void ControlWindow::T_digits_cb(Gtk::Adjustment *adj)
{
    gfloat val=adj->get_value();

    spinner_T_val->set_digits ((gint)val);
    (spinner_T_val->get_adjustment())->set_step_increment(pow(10, (int)-val));
}

void ControlWindow::he_digits_cb(Gtk::Adjustment *adj)
{
    gfloat val=adj->get_value();

    spinner_he_val->set_digits ((gint)val);
    (spinner_he_val->get_adjustment())->set_step_increment(pow(10, (int)-val));
	if (rb_constant.get_active()) { h_digits=(gint)val; }
	if (rb_rndh.get_active()) { e_digits=(gint)val; }
}

void ControlWindow::p_digits_cb(Gtk::Adjustment *adj)
{
    gfloat val=adj->get_value();

    spinner_p_val->set_digits ((gint)val);
    (spinner_p_val->get_adjustment())->set_step_increment(pow(10, (int)-val));
}

void ControlWindow::refresh_size_cb(Gtk::Adjustment *adj)
{
    refresh_size_value=adj->get_value();
    data_display->refresh_size_update();
}

void ControlWindow::start_connection() {
    	if (data_display->connect_refresh_points_with_delay==1) {
connection=Gtk::Main::timeout.connect(SigC::slot(this,&ControlWindow::refresh_points_with_delay), refresh_delay*500); }
    	if (data_display->connect_refresh_points_without_delay==1) {
connection=Gtk::Main::idle.connect(SigC::slot(this,&ControlWindow::refresh_points_without_delay)); }
	data_display->set_connected_true();
}

void ControlWindow::end_connection() {
    connection.disconnect(); 
	data_display->set_connected_false();
}

void ControlWindow::refresh_delay_cb(Gtk::Adjustment *adj)
{
    gfloat val=adj->get_value();

    refresh_delay=(int)val;
    
    if (refresh_delay>0) {
	/* the delay is not well handled with timeout, hence I 
	implement another mechanism inside refresh_points...*/
    	data_display->connect_refresh_points_with_delay=1;
    	data_display->connect_refresh_points_without_delay=0;
    } else {
    	data_display->connect_refresh_points_without_delay=1;
    	data_display->connect_refresh_points_with_delay=0;
    }
    start_connection();
}

/***********************************************************************************
   DataDisplay
************************************************************************************/

void DataDisplay::set_connected_false() {
	connected=false;
	lattice_window.drawing_area.set_connected_true();
}

void DataDisplay::set_connected_true() {
	connected=true;
	lattice_window.drawing_area.set_connected_true();
}

void DataDisplay::start_connection() {
	if (!connected && control_window_ready) 
		control_window->start_connection();
}

void DataDisplay::end_connection() {
	if (connected) 
		control_window->end_connection();
}

DataDisplay::DataDisplay() : lattice_window(this) {
  mtrand.seed( time(NULL));
  connected=false;
  control_window_ready=false;
}

DataDisplay::~DataDisplay() {
}

int DataDisplay::refresh_points_without_delay() {
    GdkRectangle update_rect;

	refresh_number++;

	static int x=0, y=0;
	static int width=0, height=0;

	if (control_window->rb_random.get_active()) {
		x= mtrand.randInt(hlength-refresh_hsize);
		y= mtrand.randInt(vlength-refresh_vsize);
		width=refresh_hsize; height=refresh_vsize;
	} else if (control_window->rb_sweep.get_active()) {
		if (x+width<hlength) {
			x+=refresh_hsize;
			if (x+refresh_hsize>hlength) width=hlength-x;
			else width=refresh_hsize;
		} else { x=0; width=refresh_hsize; 
			if (y+height<vlength) {
				y+=refresh_vsize;
				if (y+refresh_vsize>vlength) height=vlength-y;
				else height=refresh_vsize;
			} else {
				y=0; height=refresh_vsize;	
			}
		}
/* this is to take care of what happens in case of a resize:
	the old values of y and height might be incompatible!! */
		if (y+height>vlength) { y=0; height=refresh_vsize; }
		}
	update_rect.x = (int)x;
	update_rect.y = (int)y;
	update_rect.width = width;
	update_rect.height = height;

	if (control_window->rb_noise.get_active()) {

	if (control_window->rb_client.get_active()) {

	for (int i=x; i<x+width; i++) {
		for (int j=y; j<y+height; j++) {
			if (mtrand.rand()<prob) {
				(lattice_window.drawing_area).put_black_pixel(i,j);
			} else { (lattice_window.drawing_area).put_white_pixel(i,j); }  
	}}

	} else if (control_window->rb_server.get_active()) {

	for (int i=x; i<x+width; i++) {
		for (int j=y; j<y+height; j++) {
			if (mtrand.rand()<prob) {
				(lattice_window.drawing_area).draw_black_point(i,j);
			} else { (lattice_window.drawing_area).draw_white_point(i,j); }  

	} }
	
	} 

	} else if (control_window->rb_algo.get_active()) {

	if (control_window->rb_client.get_active()) {

	for (int i=x; i<x+width; i++) {
		for (int j=y; j<y+height; j++) {
			if (algo.site_status(i,j)) {
				(lattice_window.drawing_area).put_white_pixel(i,j);
			} else { (lattice_window.drawing_area).put_black_pixel(i,j); }  

	} }

	} else if (control_window->rb_server.get_active()) {

	for (int i=x; i<x+width; i++) {
		for (int j=y; j<y+height; j++) {
			if (algo.site_status(i,j)) {
				(lattice_window.drawing_area).draw_white_point(i,j);
			} else { (lattice_window.drawing_area).draw_black_point(i,j); }  

	} }

	} } 

	if (control_window->rb_client.get_active()) {
		(lattice_window.drawing_area).draw_image(x,y,width,height); }
		(lattice_window.drawing_area).draw(&update_rect);

	return connect_refresh_points_without_delay;
}


int DataDisplay::refresh_points_with_delay() 
{
    GdkRectangle update_rect;

	static time_t time1=time(NULL), time2;

	time2=time(NULL);
	float interval=(float)difftime(time2,time1);

	if (interval<refresh_delay) { 
		return connect_refresh_points_with_delay;
	}

	time1=time2;

    refresh_number++;

    int x= mtrand.randInt(hlength-refresh_hsize);
    int y= mtrand.randInt(vlength-refresh_vsize);
    update_rect.x = (int)x;
    update_rect.y = (int)y;
    update_rect.width = refresh_hsize;
    update_rect.height = refresh_vsize;
	if (control_window->rb_noise.get_active()) {

	if (control_window->rb_client.get_active()) {

	for (int i=x; i<x+refresh_hsize; i++) {
		for (int j=y; j<y+refresh_vsize; j++) {
			if (mtrand.rand()<prob) {
				(lattice_window.drawing_area).put_black_pixel(i,j);
			} else { (lattice_window.drawing_area).put_white_pixel(i,j); }  
	}}

	} else if (control_window->rb_server.get_active()) {

	for (int i=x; i<x+refresh_hsize; i++) {
		for (int j=y; j<y+refresh_vsize; j++) {
			if (mtrand.rand()<prob) {
				(lattice_window.drawing_area).draw_black_point(i,j);
			} else { (lattice_window.drawing_area).draw_white_point(i,j); }  

	} }
	
	} 

	} else if (control_window->rb_algo.get_active()) {

	if (control_window->rb_client.get_active()) {

	for (int i=x; i<x+refresh_hsize; i++) {
		for (int j=y; j<y+refresh_vsize; j++) {
			if (algo.site_status(i,j)) {
				(lattice_window.drawing_area).put_white_pixel(i,j);
			} else { (lattice_window.drawing_area).put_black_pixel(i,j); }  

	} }

	} else if (control_window->rb_server.get_active()) {

	for (int i=x; i<x+refresh_hsize; i++) {
		for (int j=y; j<y+refresh_vsize; j++) {
			if (algo.site_status(i,j)) {
				(lattice_window.drawing_area).draw_white_point(i,j);
			} else { (lattice_window.drawing_area).draw_black_point(i,j); }  

	} }

	} } 

	if (control_window->rb_client.get_active()) {
		(lattice_window.drawing_area).draw_image(x,y,refresh_hsize,refresh_vsize); }
		(lattice_window.drawing_area).draw(&update_rect);

	return connect_refresh_points_with_delay;
}

void DataDisplay::connect_with_delay() {
//Gtk::Main::timeout.connect(SigC::slot(this,&DataDisplay::refresh_points_with_delay), 
//refresh_delay*500); 
}

void DataDisplay::connect_without_delay() {
//Gtk::Main::idle.connect(SigC::slot(this,&DataDisplay::refresh_points_without_delay));
//Gtk::Main::idle.connect(slot(this,&DataDisplay::stop));
}

void DataDisplay::init() {
	time_start=time(NULL);
	if (control_window_ready)
	control_window->set_button_states(true,true,false, false);
	algo.init(hlength, vlength, prob, temp, field, epsi);
}

void DataDisplay::run() {
    if (control_window_ready) {
    algo_t al=Glauber;
    if (control_window->rb_glauber.get_active()) { 
    	if (control_window->ck_field.get_active()) { 
    		if (control_window->rb_constant.get_active())  
			{ al=Glauber_with_field; }
		else if (control_window->rb_rndh.get_active())  
			{ al=Glauber_with_random_field; }
	} else { al=Glauber; }
    } else if (control_window->rb_kawasaki.get_active()) { al=Kawasaki; }
    int num=1;
    if (control_window->ck_bipro.get_active()) { num=2; }
    else { num=1; }
    launch_computation_thread(al,num);
    control_window->set_button_states(false,false,true,true);
    control_window->statistics();
    }
}

void DataDisplay::restart() {
	stop();
	init();
	run();
}

void DataDisplay::stop() {
    if (nthreads>0) { stop_computation_threads(); 
    	control_window->set_button_states(true,true,false,false);}
}

void DataDisplay::erase() {
	lattice_window.drawing_area.erase();
}

void DataDisplay::show() {
	lattice_window.show();
}

void DataDisplay::set_black_color() {
	lattice_window.drawing_area.set_black_color(); 
}

void DataDisplay::set_white_color() {
	lattice_window.drawing_area.set_white_color(); 
}

void DataDisplay::refresh_size_update()
{
    if (refresh_size_value<30) { 
	refresh_hsize= (int)refresh_size_value;
	refresh_vsize= (int)refresh_size_value;
    } else if (refresh_size_value>70) {
	refresh_hsize= hlength-100+(int)refresh_size_value;
	refresh_vsize= vlength-100+(int)refresh_size_value;
    } else  {
	    refresh_hsize=30+((int)refresh_size_value-30)*(hlength-60) / 40;
	    refresh_vsize=30+((int)refresh_size_value-30)*(vlength-60) / 40;
    }
}


}

int main (int argc, char *argv[])
{
	cout << "

    Gnu-Ising is a program to simulate the 2D stochastic Ising model.
    Copyright (C) 2001 Raphael Cerf

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    E-mail contact: Raphael.Cerf@math.u-psud.fr
    Ordinary mail: Raphael Cerf, Mathematique, batiment 425,
	Universite Paris Sud, 91405 Orsay Cedex

	";
	

  Gtk::Main app(argc,argv);
  interface::DataDisplay data_display;
  interface::ControlWindow control_window(&data_display);

  app.run();
  return 0;
}

